
#include "elementary.h"

#include <vector>
#include <iostream>

std::vector<std::vector<bool> > elementary(unsigned char rule, int iterations, int width)
{
  // Set defaults if missing
  (iterations == 0) && (iterations = 24);
  (width == 0) && (width = (2 * iterations) + 1);

  std::vector<std::vector<bool> > result;


  // Generate ruleset
  bool ruleset[8] = {0};
  unsigned char mask = 1;
  
  for (int i = 0; i < 8; ++i)
  {
    ruleset[i] = rule & mask;
    mask = mask << 1;
  }


  // Initialize the automaton
  std::vector<bool> automaton;
  std::vector<bool> next_t;
  for (int i = 0; i < width; ++i)
  {
    automaton.push_back(false);
    next_t.push_back(false);
  }
  automaton[width / 2] = true;


  // First row
  result.push_back(automaton);


  // Iterate
  for (int i = 1; i < iterations; ++i)
  {
    for (int j = 0; j < width; ++j)
    {
      unsigned char offset = 0;
//      offset += (automaton[(j + width - 1) % width] ? 4 : 0);
//      offset += (automaton[j] ? 2 : 0);
//      offset += (automaton[(j + 1) % width] ? 1 : 0);
      
      
      for (int k = -1; k <= 1; ++k)
      {
        offset = offset << 1;
        offset ^= automaton[(width + j + k) % width];
      }
      
      next_t[j] = ruleset[offset];
    }

    automaton = next_t;
    result.push_back(automaton);
  }  

  return result;
}

