
#include "graymap.h"

#include <vector>
#include <string>
#include <fstream>

#include <iostream>

void Graymap::clear()
{
  graymap.clear();
}


void Graymap::addLine(std::vector<unsigned short> line)
{
  // Fit line to size if needed…
  if (!graymap.empty())
  {
    while (line.size() < graymap[0].size())
    {
      line.push_back(GRAYMAP_MAX_VALUE);
    }
    while (line.size() > graymap[0].size())
    {
      line.pop_back();
    }
  }
  
  // Add line
  graymap.push_back(line);
}


void Graymap::setSize(int width, int height)
{
  setHeight(height);
  setWidth(width);
}


void Graymap::setWidth(int width)
{
  for (std::vector<std::vector<unsigned short> >::iterator row = graymap.begin();
      row != graymap.end(); ++row)
  {
    while (row->size() < width) { row->push_back(GRAYMAP_MAX_VALUE); }
    while (row->size() > width) { row->pop_back(); }
  }
}


void Graymap::setHeight(int height)
{
  while (graymap.size() > height) { graymap.pop_back(); }
  
  if (graymap.size() < height)
  {
    int width = 0;
    std::vector<unsigned short> new_row;
    
    if (graymap.size() > 0) { width = graymap[0].size(); }
    for (int i = 0; i < width; ++i) { new_row.push_back(GRAYMAP_MAX_VALUE); }
    while (graymap.size() < height) { graymap.push_back(new_row); }
  }
}


unsigned short Graymap::getPixel(int x, int y)
{
  if (isInside(x, y))
  {
    return graymap[y][x];
  } else
  {
    return 0;
  }
}


void Graymap::setPixel(int x, int y, unsigned short value)
{
  if (isInside(x, y))
  {
    graymap[y][x] = value;
  }
}


void Graymap::writePGM(std::string filename, bool normalized)
{
  int height = graymap.size();
  if (!height) return;
  int width = graymap[0].size();
  if (!width) return;

  std::ofstream file;
  file.open(filename.c_str());

  file << "P2" << std::endl;
  file << width << " " << height << std::endl;
  if (normalized) { file << value_span() << std::endl; }
  else { file << GRAYMAP_MAX_VALUE << std::endl; }

  for (std::vector<std::vector<unsigned short> >::iterator row = graymap.begin();
      row != graymap.end(); ++row)
  {
    for (std::vector<unsigned short>::iterator pixel = row->begin();
        pixel != row->end(); ++pixel)
    {
      if (normalized) { file << (int)*pixel - value_min() << " "; }
      else { file << (int)*pixel << " "; }
    }
    file << std::endl;
  }

  file.close();
}


unsigned short Graymap::value_span()
{
  return value_max() - value_min();
}

unsigned short Graymap::value_max()
{
  unsigned short max = 0;
  for (std::vector<std::vector<unsigned short> >::iterator row = graymap.begin();
      row != graymap.end(); ++row)
  {
    for (std::vector<unsigned short>::iterator cell = row->begin();
        cell != row->end(); ++cell)
    {
      max = (*cell > max ? *cell : max);
    }
  }
  return max;
}

unsigned short Graymap::value_min()
{
  unsigned short min = GRAYMAP_MAX_VALUE;
  for (std::vector<std::vector<unsigned short> >::iterator row = graymap.begin();
      row != graymap.end(); ++row)
  {
    for (std::vector<unsigned short>::iterator cell = row->begin();
        cell != row->end(); ++cell)
    {
      min = (*cell < min ? *cell : min);
    }
  }
  return min;
}

void Graymap::print()
//FIXME At the moment ascii printing is binary, just as for the bitmap.
{
  for (std::vector<std::vector<unsigned short> >::iterator row = graymap.begin();
      row != graymap.end(); ++row)
  {
    std::cout << "|";
    for (std::vector<unsigned short>::iterator cell = row->begin();
        cell != row->end(); ++cell)
    {
      std::cout << (*cell ? "*" : " ");
    }
    std::cout << "|" << std::endl;
  }
}


std::vector<std::vector<unsigned short> > Graymap::getGraymap()
{
  return graymap;
}


bool Graymap::isInside(int x, int y)
{
  if (y < 0 or y >= graymap.size()) { return false; }
  if (x < 0 or x >= graymap[y].size()) { return false; }
  return true;
}

