#include "ca.h"
#include "convert.h"

#include <stdint.h>
#include <iostream>
#include <cmath>

CA::CA(uint32_t ca_size, uint64_t rule_number,
    uint32_t rule_radius, uint32_t base)
{
  // Initialization
  size = ca_size;
  this->base = base;
  this->rule_number = rule_number;
  this->rule_radius = rule_radius;

  // Initialize state
  extended_state = new uint32_t[size + 2 * rule_radius];
  for (uint32_t i = 0; i < (size + 2 * rule_radius); ++i)
  {
    extended_state[i] = 0;
  }
  state = extended_state + rule_radius;

  // Create rule table
  rule_table_size = std::pow(base, (2 * rule_radius + 1));
  rule_table = new uint32_t[rule_table_size];
  number2array(rule_number, base, rule_table_size, rule_table, true);
}

CA::~CA()
{
  if (extended_state != NULL)
    delete[] extended_state;
  if (rule_table != NULL)
    delete[] rule_table;
}

uint32_t * const CA::ptr()
{
  // get pointer to state
  return state;
}

uint32_t * const CA::next()
{
  // next step & get ptr to state
  uint32_t next_state[size];
  for (int i = 0; i < size; ++i) 
  {
    uint64_t index = 0;
    array2number(index, base, 2 * rule_radius + 1, &extended_state[i], false);
    next_state[i] = rule_table[index];
  }

  for (int i = 0; i < size; ++i)
  {
    state[i] = next_state[i];
  }

  fill_edges();
  
  return state;
}

void CA::set_state(uint32_t *new_state)
{
  // set state
  if (state != NULL)
  {
    for (uint32_t i = 0; i < size; ++i)
    {
      state[i] = new_state[i];
    }
  }
  fill_edges();
}

void CA::print()
{
  // Print state to stdout
  for (uint32_t i = 0; i < (size-1); ++i)
  {
    std::cout << state[i] << " ";
  }
  std::cout << state[size-1] << std::endl;
}


// This funciton enforces periodic boundary conditions.
void CA::fill_edges()
{
  // Go forwards to end
  for (int i = rule_radius; i < 2 * rule_radius; ++i)
  {
    extended_state[i + size] = extended_state[i];
  }
  // Go backwards from start
  for (int i = 0; i < rule_radius; ++i)
  {
    extended_state[i] = extended_state[i + size];
  }
}

