#include "convert.h"
#include "necklace.h"
#include "ca.h"
#include "transform.h"
#include "assignment.h"
#include "simulation.h"

#include <iostream>
#include <stdint.h>
#include <vector>
#include <set>
#include <queue>
#include <fstream>

#include <thread>
#include <chrono>

#include <cstdlib>
#include <ctime>
#include <cmath>

#define MIN_SIZE 1
#define MAX_SIZE 24
#define BASE 2
#define RULE 122
#define RADIUS 2
//#define WINDOW_SIZES {1, 2, 3, 5, 6, 7, 9, 10, 11, 12, 13, 14, 15}
//#define WINDOW_SIZES {17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31}
//#define WINDOW_SIZES {4, 8, 16, 32}
#define WINDOW_SIZES {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, \
17, 18, 19, 20, 21, 22, 23, 24}
#define FULL_PRINT false
#define THREADS 10
#define ASSIGNMENT_COUNT 100

uint64_t ECA_rules[88]
      = {  0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,
          11,  12,  13,  14,  15,  18,  19,  22,  23,  24,  25,
          26,  27,  28,  29,  30,  32,  33,  34,  35,  36,  37,
          38,  40,  41,  42,  43,  44,  45,  46,  50,  51,  54,
          56,  57,  58,  60,  62,  72,  73,  74,  76,  77,  78,
          90,  94, 104, 105, 106, 108, 110, 122, 126, 128, 130,
         132, 134, 136, 138, 140, 142, 146, 150, 152, 154, 156,
         160, 162, 164, 168, 170, 172, 178, 184, 200, 204, 232};

void populate_queue(std::queue<Assignment> &assignment_queue, uint64_t rule)
{
    for (uint32_t size = MIN_SIZE; size <= MAX_SIZE; ++size)
    {
      Assignment temp =
      {
        BASE, // base
        RADIUS, // radius
        rule, // rule #
        size, // ca size
        WINDOW_SIZES, // window_sizes
        {}, // results
        false // finished flag
      };
      assignment_queue.push(temp);
    }
}

int main(int argc, char *argv[])
{
  std::thread simulators[THREADS]; // Simulators
  Assignment assignments[THREADS]; // Assignments
  std::queue<Assignment> assignment_queue;

  std::ofstream result_file;
  result_file.open("results.sql");

  int32_t running_simulations_count = 0;

  // Populate queue of assignments
  if (BASE == 2 && RADIUS == 1) // Use full listing for ECAs.
  {
    std::cout << "Generating 88 ECA assignments..." << std::endl;
    for (uint32_t rule_index = 0; rule_index < 88; ++rule_index)
    {
      populate_queue(assignment_queue, ECA_rules[rule_index]);
    }
  }
  else // Generate random assignments for non-ECAs.
  {
    std::cout << "Generating " << ASSIGNMENT_COUNT << " assignments..." << std::endl;
    std::srand(std::time(0));
    uint64_t rule_max = pow(BASE, pow(BASE, (RADIUS * 2) + 1));
    for (int i = 0; i < ASSIGNMENT_COUNT; ++i)
    {
      uint64_t rule = std::rand() % rule_max;
      populate_queue(assignment_queue, rule);
    }
  }

  // Fill up up to THREADS threads, start them and set them free!
  for (int i = 0; i < THREADS; ++i)
  {
    if (!assignment_queue.empty())
    {
      assignments[i] = assignment_queue.front();
      assignment_queue.pop();
      simulators[i] = std::thread(simulate, &assignments[i]);
      simulators[i].detach();
      ++running_simulations_count;
    }
  }

  // Catch finished threads, and start new ones
  while(running_simulations_count > 0)
  {
    for (int i = 0; i < THREADS; ++i)
    {
      // Is the job finished?
      if (assignments[i].finished == true)
      {
        --running_simulations_count;
        // Handle the result
        for (std::vector<Datapoint>::iterator it = assignments[i].results.begin();
            it != assignments[i].results.end(); ++it)
        {
          result_file << to_sql_string(&(*it)) << std::endl;
          assignments[i].finished = false;
        }

        // Start new job if available
        if (!assignment_queue.empty())
        {
          assignments[i] = assignment_queue.front();
          assignment_queue.pop();
          simulators[i] = std::thread(simulate, &assignments[i]);
          simulators[i].detach();
          ++running_simulations_count;
        }
        std::cout << assignment_queue.size() << " jobs in queue, "
            << running_simulations_count << " processing." << std::endl;
      }
    }
    // Sleep some...
    std::this_thread::sleep_for(std::chrono::milliseconds(1000));
  }

  result_file.close();

  return 0;
}

