#include "necklace.h"

#include <iostream>


Necklace::Necklace(uint32_t size, uint32_t base)
{
  this->size = size;
  this->base = base;

  state = new uint32_t[size];
  state1index = state - 1;

  for (uint32_t i = 0; i < size; ++i)
  {
    state[i] = (base - 1);
  }
}

Necklace::~Necklace()
{
  delete[] state;
}

uint32_t * const Necklace::ptr()
{
  // get pointer to necklace
  return state;
}

uint32_t * const Necklace::next()
{
  // Increment necklace.
  // Based on algorithm described by Fredricksen and Kessler (1986).
  uint32_t j;
  do
  {
    // Find j
    j = size;
    while (j > 0 && state1index[j] == 0)
    {
      --j;
    }
    
    // special case: go from 0^size to (base-1)^size
    if (j == 0)
    {
      for (int i = 0; i < size; ++i)
      {
        state[i] = (base - 1);
      }
      return state;
    }

    // Decrement position j
    --state1index[j];
    
    // Fill positions after j
    for (int i = j; i < size; ++i)
    {
      state[i] = state[i-j];
    }
  } while (size % j != 0);


  // Return pointer to necklace
  return state;
}

bool Necklace::atstart()
{
  return (state[size-1] == (base-1));
}

void Necklace::print()
{
  for (int i = 0; i < (size - 1); ++i)
  {
    std::cout << state[i] << " ";
  }
  std::cout << state[size-1] << std::endl;
}

