#include "simulation.h"

#include "ca.h"
#include "necklace.h"
#include "transform.h"
#include "datapoint.h"

#include <stdint.h>
#include <set>

void simulate(Assignment *a)
{
  // Find the required time domain size for the spacetime plot
  uint32_t window_size = 0;
  for (std::vector<uint32_t>::iterator it = a->window_sizes.begin();
      it != a->window_sizes.end(); ++it)
  {
    if (*it > window_size)
    {
      window_size = *it;
    }
  }

  // Spacetime plot, reused for each evolution of the CA
  uint32_t spacetime[a->size][window_size];

  // CA
  CA ca(a->size, a->rule_number, a->rule_radius, a->base);
  uint32_t *ca_state = ca.ptr();

  // Sets of spectra, for counting
  std::set<std::vector<uint32_t> > spectersets[a->window_sizes.size()];

  // Necklace
  Necklace necklace(a->size, a->base);

  // Run for all necklaces (all initial states)
  do
  {
    ca.set_state(necklace.ptr());
    necklace.next();

    // Fill spacetime plot
    for (int i = 0; i < window_size; ++i)
    {
      // Spacetime plot
      for (int j = 0; j < a->size; ++j)
      {
        spacetime[j][i] = ca_state[j];
      }
      // next state
      ca.next();
    }

    // Generate toggle spectra
    for (uint32_t i = 0; i < a->window_sizes.size(); ++i)
    {
      // Generate specter
      uint32_t window_size = a->window_sizes[i];
      uint32_t toggle_specter[window_size];
      for (uint32_t j = 0; j < window_size; ++j)
      {
        toggle_specter[j] = 0;
      }
      for (uint32_t j = 0; j < a->size; ++j)
      {
        togglecount(spacetime[j], window_size, toggle_specter);
      }
      // Add specter to correct specterset
      spectersets[i].insert(
          std::vector<uint32_t>(toggle_specter, toggle_specter + window_size));
    }

  } while (!necklace.atstart());

  // Save results
  for (uint32_t i = 0; i < a->window_sizes.size(); ++i)
  {
    Datapoint datapoint =
    {
      a->base, // base
      1, // dimensions
      a->size, // size
      "", // neighbourhood
      "cyclic", // border conditions
      a->rule_radius, // radius
      a->rule_number, // rule
      "togglecount", // transformation
      a->window_sizes[i], // window size
      0, // window start position
      spectersets[i].size() // simulation result
    };
    a->results.push_back(datapoint);
  }

  // Mark as finished
  a->finished = true;
}

