#pragma once

#include <stdint.h>
#include <sqlite3.h>
#include <string>
#include <vector>
#include <utility>

// Fetch list of rules with chosen parameters - full version
std::vector<uint64_t> fetch_rulelist(
    sqlite3 *db, uint32_t cellstates, uint32_t dimensions, uint32_t size,
    std::string neighbourhood, uint32_t range, std::string boundarycondition,
    std::string transformation, uint32_t windowsize, uint32_t startposition);

// Fetch list of rules with chosen parameters - short version
std::vector<uint64_t> fetch_rulelist(
    sqlite3 *db, uint32_t cellstates, uint32_t size, uint32_t range,
    uint32_t windowsize);


// Fetch spectral diversities over varying ca size - full
std::vector<std::pair<uint32_t, uint32_t> > fetch_over_casizes(
    sqlite3 *db, uint64_t rule, uint32_t cellstates, uint32_t dimensions,
    std::string neighbourhood, uint32_t range, std::string boundarycondition,
    std::string transformation, uint32_t windowsize, uint32_t startposition);

// Fetch spectral diversities over varying ca size - short
std::vector<std::pair<uint32_t, uint32_t> > fetch_over_casizes(
    sqlite3 *db, uint64_t rule, uint32_t cellstates, uint32_t range,
    uint32_t windowsize);


// Fetch spectral diversities over varying window size - full
std::vector<std::pair<uint32_t, uint32_t> > fetch_over_winsizes(
    sqlite3 *db, uint64_t rule, uint32_t cellstates, uint32_t size,
    uint32_t dimensions, std::string neighbourhood, uint32_t range,
    std::string boundarycondition, std::string transformation,
    uint32_t startposition);

// Fetch spectral diversities over varying window size - short
std::vector<std::pair<uint32_t, uint32_t> > fetch_over_winsizes(
    sqlite3 *db, uint64_t rule, uint32_t cellstates, uint32_t size,
    uint32_t range);


// Fetch spectral diversities over varying rules - full
std::vector<std::pair<uint64_t, uint32_t> > fetch_over_rules(
    sqlite3 *db, uint32_t cellstates, uint32_t size, uint32_t dimensions,
    std::string neighbourhood, uint32_t range, std::string boundarycondition,
    std::string transformation, uint32_t startposition, uint32_t windowsize);

// Fetch spectral diversities over varying rules - short
std::vector<std::pair<uint64_t, uint32_t> > fetch_over_rules(
    sqlite3 *db, uint32_t cellstates, uint32_t size, uint32_t range,
    uint32_t windowsize);



// NOTES
//
// Short versions
// 
// DB handle            db
// (rule)               (chosen)
// base                 chosen
// size                 varying
// range                chosen
// window size          varying
//
// Full versions
//
// DB handle            db
// (rule)               (chosen)
// base                 chosen
// dimensions           constant (1)
// size                 varying
// neighbourood         constant ("")
// range                chosen
// boundary condition   constant ("cyclic")
// transformation       constant ("togglecount")
// window size          varying
// start position       constant (0)

