#include <iostream>
#include <vector>
#include <string>
#include <sstream>

#include <stdint.h>
#include <getopt.h>
#include <stdlib.h>

static int32_t graphmode;

#define SCATTER_GRAPH 0
#define LINE_GRAPH 1

#define DEFAULT_GRAPH_TYPE LINE_GRAPH

std::string header()
{
  std::stringstream result;

  result << "\\begin{tikzpicture}" << std::endl;
  result << "  \\begin{axis}[" << std::endl;
  result << "    legend pos=north west" << std::endl;
  result << "  ]" << std::endl;

  return result.str();
}

std::string footer()
{
  std::stringstream result;

  result << "  \\end{axis}" << std::endl;
  result << "\\end{tikzpicture}" << std::endl;

  return result.str();
}

std::string pointstring(std::string line1, std::string line2)
{
  std::stringstream result;

  std::stringstream sline1(line1);
  std::stringstream sline2(line2);

  uint64_t first;
  uint64_t second;

  while (sline1 >> first, sline2 >> second)
  {
    result << "(" << first << ", " << second << ")";
  }

  return result.str();
}


void usage(const char *program_name)
{
  std::cout << "  Usage: " << program_name << " [OPTIONS]" << std::endl;
  std::cout << "" << std::endl;
  std::cout << "  Options" << std::endl;
  std::cout << "  --help           display this help message" << std::endl;
  std::cout << "  --scatter        scatter plot mode" << std::endl;
  std::cout << "  --line           line plot mode (default)" << std::endl;
}

int main(int argc, char **argv)
{
  // Defaults
  graphmode = DEFAULT_GRAPH_TYPE;

  // Command line options
  int c;
  while (1)
  {
    static struct option long_options[] =
    {
      {"help", no_argument, 0, 'h'},
      {"scatter", no_argument, &graphmode, SCATTER_GRAPH},
      {"line",    no_argument, &graphmode, LINE_GRAPH},
      {0, 0, 0, 0}
    };

    int option_index = 0;
    c = getopt_long (argc, argv, "h", long_options, &option_index);

    if (c == -1)
      break;

    switch(c)
    {
      case 0:
        break;
      default:
        usage(argv[0]);
        exit(0);
        break;
    }
  }

  std::cout << header();

  std::string line1, line2;
  switch (graphmode)
  {
    case LINE_GRAPH:
      // Read data and generate coordinate lines
      while (std::getline(std::cin, line1), std::getline(std::cin, line2))
      {
        std::cout << "    \\addplot+[mark=none,color=white!85!black,solid]"
            << " plot coordinates {"
            << pointstring(line1, line2)
            << "};"
            << std::endl;
      }
      break;
    
    case SCATTER_GRAPH:
      // Read data and generate coordinate lines
      while (std::getline(std::cin, line1), std::getline(std::cin, line2))
      {
        std::cout << "    \\addplot+[only marks,mark=+]"
            << " plot coordinates {"
            << pointstring(line1, line2)
            << "};"
            << std::endl;
      }
      break;
 
  }

  std::cout << footer();

  return 0;
}

